//                                                        
//   Programa elaborado por Genaro Juarez Martinez        
//   e-mail: genaro@enigma.red.cinvestav.mx               
//               genarojm@correo.unam.mx                             
//   Area: Automatas Celulares                            
//   CINVESTAV IPN                                        
//                                                        

#import "life2dvn.h"
#define LINES 600
#define RING 600
#define FACTOR 8
#define NUM 50
int longx,longy;
int arreglo[LINES/FACTOR][RING/FACTOR],inicial[LINES/FACTOR][RING/FACTOR],auxiliar[RING/FACTOR],matriz_aux[4][8];
int iteraciones;
int neighborhood[8];
int FLAG;
int ring,lines,lineas;

/*Convierte decimal en binario*/
int rule_binary(int ruleaux)
{
    int modulo;
    int divisor;
    int i=0;
    do
      {
          modulo = fmod(ruleaux,2);
          divisor = ruleaux/2;
          neighborhood[i] = modulo;
          ruleaux = divisor;
          i++;
      }while(i<8);
    return 0;
}

/*Reviza frontera superior*/
int check_sup(int k)
{
    int help;
    help=k;
    if(k<0)
        help=ring-1;
    if(k>ring-1)
        help=k-ring;
    return(help);
}

/*Reviza frontera inferior*/
int check_inf(int l)
{
    if(l>ring-1)
        l -= ring;
    return(l);
}

/*Reviza frontera izquierda*/
int check_izq(int k)
{
    int help;
    help=k;
    if(k<0)
        help=ring-1;
    if(k>ring-1)
        help=k-ring;
    return(help);
}

/*Reviza frontera derecha*/
int check_der(int l)
{
    if(l>ring-1)
        l -= ring;
    return(l);
}

@implementation life2dvn

/*Inicializa la grafica*/
- initWithFrame:(NSRect)rects
{
    int i,j;
    [super initWithFrame:rects];
    [self allocateGState];
    FLAG=FACTOR;
    ring=RING/FACTOR;
    lines=LINES/FACTOR;
    iteraciones=0;
    /*Limpiar arreglos*/
    for(i=0;i<lines;i++)
    {
        for(j=0;j<ring;j++)
        {
            inicial[i][j]=0;
            arreglo[i][j]=0;
        }
    }
    for(i=0;i<ring;i++)
        auxiliar[i]=0;
    /* Listen to interesting notifications */
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appWillHide:) name:NSApplicationWillHideNotification object:[NSApplication sharedApplication]];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowWillMiniaturize:) name:NSWindowWillMiniaturizeNotification object:nil];
    running = NO;
    return self;
}

/*Dibuja la grafica*/
-(void)drawRect:(NSRect)rects
{
    /*Limites del cuadro*/
    NSRect limites=[self bounds];
    /*Largo y ancho del cuadro y el incremento en radianes*/
    longx=limites.size.width;
    longy=limites.size.height;
    [[NSColor whiteColor] set];
    NSRectFill([self bounds]);
    [self dibujaMalla:self];
}

- (void)dealloc
{
    /* Stop the timer */
    if (running)
    {
        [linesTimer invalidate];
        [linesTimer release];
    }
    /* Remove self as observer */
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
}

- (void)toggleRun:(id)sender
{
    /* start or stop the timer (we're called by a two-state button) */
    if (running)
   {
        [linesTimer invalidate];
        [linesTimer release];
        running = NO;
    }
    else
    {
      /* Call the animation method as often as possible... */
        linesTimer = [[NSTimer scheduledTimerWithTimeInterval:0.0 target:self selector:@selector(animate:) userInfo:nil repeats:YES] retain];
        running = YES;
    }
}

- (void)animate:(NSTimer *)timer
{
    int i,j;
    int identificador,cociente,modulo;
    for(i=0;i<lines;i++)
    {
        for(j=0;j<ring;j++)
        {
            identificador=(pow(2,0)*inicial[i-1][j]) + (pow(2,1)*inicial[i][j-1]) + (pow(2,2)*inicial[i][j]) + (pow(2,3)*inicial[i][j+1]) + (pow(2,4)*inicial[i+1][j]);
            cociente=identificador/8;
            modulo=fmod(identificador,8);
            arreglo[i][j]=matriz_aux[cociente][modulo];
        }
    }
    /*Actualiza la siguiente configuracion*/
    for(i=0;i<lines;i++)
        for(j=0;j<ring;j++)
            inicial[i][j]=arreglo[i][j];
    for(i=0;i<lines;i++)
        for(j=0;j<ring;j++)
            arreglo[i][j]=0;
    /*Dibuja la nueva configuracion*/
    [self dibujaConfiguracion:self];
    iteraciones++;
}

- (void)windowWillMiniaturize:(NSNotification *)notification
{
    if (running && ([notification object] == [self window]))
        [self toggleRun:nil];
}

- (void)appWillHide:(NSNotification *)notification
{
    if (running)
        [self toggleRun:nil];
}

/*Dibuja una linea*/
-(void)dibujarLinea:(int)x1:(int)y1:(int)x2:(int)y2:(float)rojo:(float)verde:(float)azul
{
    PSsetrgbcolor(rojo,verde,azul);
    PSmoveto(x1,y1);
    PSlineto(x2,y2);
}

/*Dibuja una celula*/
-(void)dibujaPunto:(int)x:(int)y:(float)rojo:(float)verde:(float)azul
{
    /*Activa la accion del cuadro */
    [self lockFocus];
    PSnewpath();
    /*Dibujar Punto*/
    PSsetrgbcolor(rojo,verde,azul);
    PSarc(x,y,2,0,360);
    PSfill();
    /*Desactiva el cuadro*/
    PSstroke();
    [self unlockFocus];
}

/*Dibuja la malla*/
-(void)dibujaMalla:(id)sender
{
    int i;
    PSnewpath();
    /*Lineas verticales*/
    for(i=0;i<RING;i+=FACTOR)
        [self dibujarLinea:i:0:i:longy:0.80:0.80:0.80];
    /*Lineas horizontales*/
    for(i=0;i<LINES;i+=FACTOR)
        [self dibujarLinea:0:i:longx:i:0.80:0.80:0.80];
    PSstroke();
}

/*Configuracion aleatoria*/
-(void)configuracionAleatoria:(id)sender
{
    int i,j;
    float y,flag;
    time_t segundos;
    iteraciones=0;
    /*Calcular valores aleatorios*/
    time(&segundos);
    srand((unsigned)(segundos % 65536));
    flag=[densidad floatValue];
    for(i=0;i<lines;i++)
    {
        for(j=0;j<ring;j++)
        {
            y = rand()/pow(2,31);
            if(y>=flag)
                inicial[i][j] = 0;
            else
                inicial[i][j] = 1;
        }
    }
    /*Dibuja la nueva configuracion*/
    [self dibujaConfiguracion:self];
}

/*Dibuja configuracion*/
-(void)dibujaConfiguracion:(id)sender
{
    int i,j,aux_x=0,aux_y=0,contador=0;
    /*Activa la accion del cuadro */
    [self lockFocus];
    /*Limpia el view*/
    [self limpiarVista:self];
    /*Dibujar la configuracion inicial*/
    for(i=0,aux_y=3;i<lines;i++,aux_y+=FACTOR)
    {
        for(j=0,aux_x=5;j<ring;j++,aux_x+=FACTOR)
        {
            if(inicial[i][j]==1)
            {
                [self dibujaPunto:aux_x:aux_y:0:0:0];
                contador++;
            }
        }
    }
    /*Pinta el momento de generar el punto*/
    [[self window] flushWindow];
    [self unlockFocus];
    /*Dice cuantas clulas estan vivas*/
    [cellvivas setIntValue:contador];
    [generaciones setIntValue:iteraciones];
}

/*Utiliza regla 110 para llenar el plano inicial*/
-(void)regla110:(id)sender
{
    int i,j,tmp;
    float y,flag;
    time_t segundos;
    /*Limpiar arreglos*/
    for(i=0;i<lines;i++)
        for(j=0;j<ring;j++)
            inicial[i][j]=0;
    for(j=0;j<ring;j++)
        auxiliar[j]=0;
    /*Calcular valores aleatorios*/
    time(&segundos);
    srand((unsigned)(segundos % 65536));
    flag=[densidad floatValue];
    for(j=0;j<ring;j++)
    {
        y = rand()/pow(2,31);
        if(y>=flag)
            inicial[0][j] = 0;
        else
            inicial[0][j] = 1;
    }
    /*Configuracion inicial lineal*/
    for(j=0;j<ring;j++)
        auxiliar[j]=inicial[0][j];
    /*Transforma la regla en binario*/
    rule_binary(110);
    /*Calcula la evolucion*/
    for(i=1;i<lines;i++)
    {
        /*Calcula la siguiente evolucion*/
        for(j=0;j<ring;j++)
        {
            tmp = auxiliar[check_inf(j-1)]*4 + auxiliar[j]*2 + auxiliar[check_sup(j+1)];
            inicial[i][j]=neighborhood[tmp];
        }
        for(j=0;j<ring;j++)
            auxiliar[j]=inicial[i][j];
    }
    /*Dibuja la nueva configuracion*/
    [self dibujaConfiguracion:self];
}

/*Traer regla de control*/
-(void)establecerRegla:(int)renglon:(int)columna:(int)estado
{
    /*Traer regla de control*/
    matriz_aux[renglon][columna]=estado;
}

/*Evolucion paso a paso*/
-(void)pasoaPaso:(id)sender
{
    int i,j;
    int identificador,cociente,modulo;
    /*Aplicar funcion de transicion*/
    for(i=0;i<lines;i++)
    {
        for(j=0;j<ring;j++)
        {
            identificador=(pow(2,0)*inicial[i-1][j]) + (pow(2,1)*inicial[i][j-1]) + (pow(2,2)*inicial[i][j]) + (pow(2,3)*inicial[i][j+1]) + (pow(2,4)*inicial[i+1][j]);
            cociente=identificador/8;
            modulo=fmod(identificador,8);
            arreglo[i][j]=matriz_aux[cociente][modulo];
        }
    }
    /*Actualiza la siguiente configuracion*/
    for(i=0;i<lines;i++)
        for(j=0;j<ring;j++)
            inicial[i][j]=arreglo[i][j];
    for(i=0;i<lines;i++)
        for(j=0;j<ring;j++)
            arreglo[i][j]=0;
    /*Dibuja la nueva configuracion*/
    [self dibujaConfiguracion:self];
    iteraciones++;
}

/*Limpia el view*/
-(void)limpiarVista:(id)sender
{
    [[NSColor whiteColor] set];
    NSRectFill([self bounds]);
    [self dibujaMalla:self];
}

/*Limpia todo*/
-(void)limpiarTodo:(id)sender
{
    int i,j;
    [self display];
    /*Limpiar arreglos*/
    for(i=0;i<lines;i++)
    {
        for(j=0;j<ring;j++)
        {
            inicial[i][j]=0;
            arreglo[i][j]=0;
        }
    }
    for(i=0;i<ring;i++)
        auxiliar[i]=0;
    iteraciones=0;
    [cellvivas setIntValue:0];
    [generaciones setIntValue:iteraciones];
    /*Dibuja la nueva configuracion*/
    [self dibujaConfiguracion:self];
}

/*Activa el mouse con mouse down*/
-(void)mouseDown:(NSEvent *)event
{
    int cuenta_multiplo=0,auxiliar1=0;
    int m_der=0,m_izq=0,m_inf=0,m_sup=0,pos_x=0,pos_y=0,coor_x=0,coor_y=0;
    [posicionx setIntValue:[event locationInWindow].x];
    [posiciony setIntValue:[event locationInWindow].y];
    /*Restringuiendo valores para X*/
    if([posicionx intValue]<=0)
        [posicionx setIntValue:5];
    if([posicionx intValue]>(longx-7))
        [posicionx setIntValue:(longx-7)];
    /*Restringuiendo valores para Y*/
    if([posiciony intValue]<=0)
        [posiciony setIntValue:5];
    if([posiciony intValue]>(longy-7))
        [posiciony setIntValue:(longy-7)];
    /*Determina el lugar adecuado de la celula para el eje X*/
    cuenta_multiplo=[posicionx intValue]/FACTOR;
    coor_x=cuenta_multiplo;
    auxiliar1=cuenta_multiplo+1;
    m_der=cuenta_multiplo*FACTOR;
    m_izq=auxiliar1*FACTOR;
    if([posicionx intValue]>=m_der && [posicionx intValue]<=m_izq)
        pos_x=m_der+5;
    /*Limpia variables de conteo*/
    cuenta_multiplo=0;
    auxiliar1=0;
    /*Determina el lugar adecuado de la celula para el eje Y*/
    cuenta_multiplo=[posiciony intValue]/FACTOR;
    coor_y=cuenta_multiplo;
    auxiliar1=cuenta_multiplo+1;
    m_inf=cuenta_multiplo*FACTOR;
    m_sup=auxiliar1*FACTOR;
    if([posiciony intValue]>=m_inf && [posiciony intValue]<=m_sup)
        pos_y=m_inf+3;
    /*Visualiza las coordenadas*/
    [posicionx setIntValue:coor_x];
    [posiciony setIntValue:coor_y];
    /*Almacena arreglo de celulas*/
    if(inicial[coor_y][coor_x]==0)
    {
        inicial[coor_y][coor_x]=1;
        /*Activa la accion del cuadro */
        [self lockFocus];
        PSnewpath();
        /*Dibujar punto*/
        PSsetrgbcolor(0,0,0);
        PSarc(pos_x,pos_y,2,0,360);
        PSfill();
        /*Desactiva el cuadro*/
        PSstroke();
        /*Pinta el momento de generar el punto*/
        [[self window] flushWindow];
        [self unlockFocus];
    }
    else
    {
        inicial[coor_y][coor_x]=0;
        /*Activa la accion del cuadro */
        [self lockFocus];
        PSnewpath();
        /*Dibujar punto*/
        PSsetrgbcolor(1,1,1);
        PSarc(pos_x,pos_y,2,0,360);
        PSfill();
        /*Desactiva el cuadro*/
        PSstroke();
        /*Pinta el momento de generar el punto*/
        [[self window] flushWindow];
        [self unlockFocus];
    }
}

@end
